from typing import List

from devicemanager.interactor.dto import DeviceStatusDto


class UpdateDevicesStatusRequest(object):
    def __init__(
        self,
        devices,    # type: List[DeviceStatusDto]
        operator,   # type: str
    ):
        # type: (...) -> None

        self.devices = devices
        self.operator = operator


