class OperatorSession(object):

    def __init__(
        self,           # type: OperatorSession
        pos_id,         # type: str
        session_id,     # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.session_id = session_id
