class LoyaltyConfigurations(object):

    def __init__(
        self,
        print_loyalty_coupon_after_fiscalization,   # type: bool
        print_loyalty_coupon_delivery_orders,       # type: bool
        text_to_print_in_loyalty_coupon,            # type: unicode
        store_name,                                 # type: unicode
    ):
        # type: (...) -> None

        self.print_loyalty_coupon_after_fiscalization = print_loyalty_coupon_after_fiscalization
        self.print_loyalty_coupon_delivery_orders = print_loyalty_coupon_delivery_orders
        self.text_to_print_in_loyalty_coupon = text_to_print_in_loyalty_coupon
        self.store_name = store_name
