from typing import List


class GoalsCache(object):
    def __init__(
        self,
        store_goals,    # type: object
        item_goals,     # type: List
    ):
        # type: (...) -> None

        self.store_goals = store_goals
        self.item_goals = item_goals
