from application.service import ChromeUpdaterService


class ChromeUpdaterInteractor(object):
    def __init__(
        self,
        chrome_updater_service  # type: ChromeUpdaterService
    ):
        # type: (...) -> None

        self.chrome_updater_service = chrome_updater_service

    def update_chrome(
        self,
        validate    # type: bool

    ):
        # type: (...) -> str

        return self.chrome_updater_service.update_chrome(
            validate_version=validate,
            restart_hv=True
        )




