# -*- coding: utf-8 -*-
from application.repository.pos import LoaderUpdaterPOSRepository


class HasPendingLoaderUpdateInteractor(object):
    def __init__(
        self,               # type: HasPendingLoaderUpdateInteractor
        pos_repository,     # type: LoaderUpdaterPOSRepository
    ):
        # type: (...) -> None

        self.pos_repository = pos_repository

    def get_loader_pending_update(
        self,   # type: HasPendingLoaderUpdateInteractor
    ):
        # type: (...) -> bool

        return self.pos_repository.has_pending_loader_to_update()
