data = (
'bbess',    # 0x00
'bbeng',    # 0x01
'bbej',    # 0x02
'bbec',    # 0x03
'bbek',    # 0x04
'bbet',    # 0x05
'bbep',    # 0x06
'bbeh',    # 0x07
'bbyeo',    # 0x08
'bbyeog',    # 0x09
'bbyeogg',    # 0x0a
'bbyeogs',    # 0x0b
'bbyeon',    # 0x0c
'bbyeonj',    # 0x0d
'bbyeonh',    # 0x0e
'bbyeod',    # 0x0f
'bbyeol',    # 0x10
'bbyeolg',    # 0x11
'bbyeolm',    # 0x12
'bbyeolb',    # 0x13
'bbyeols',    # 0x14
'bbyeolt',    # 0x15
'bbyeolp',    # 0x16
'bbyeolh',    # 0x17
'bbyeom',    # 0x18
'bbyeob',    # 0x19
'bbyeobs',    # 0x1a
'bbyeos',    # 0x1b
'bbyeoss',    # 0x1c
'bbyeong',    # 0x1d
'bbyeoj',    # 0x1e
'bbyeoc',    # 0x1f
'bbyeok',    # 0x20
'bbyeot',    # 0x21
'bbyeop',    # 0x22
'bbyeoh',    # 0x23
'bbye',    # 0x24
'bbyeg',    # 0x25
'bbyegg',    # 0x26
'bbyegs',    # 0x27
'bbyen',    # 0x28
'bbyenj',    # 0x29
'bbyenh',    # 0x2a
'bbyed',    # 0x2b
'bbyel',    # 0x2c
'bbyelg',    # 0x2d
'bbyelm',    # 0x2e
'bbyelb',    # 0x2f
'bbyels',    # 0x30
'bbyelt',    # 0x31
'bbyelp',    # 0x32
'bbyelh',    # 0x33
'bbyem',    # 0x34
'bbyeb',    # 0x35
'bbyebs',    # 0x36
'bbyes',    # 0x37
'bbyess',    # 0x38
'bbyeng',    # 0x39
'bbyej',    # 0x3a
'bbyec',    # 0x3b
'bbyek',    # 0x3c
'bbyet',    # 0x3d
'bbyep',    # 0x3e
'bbyeh',    # 0x3f
'bbo',    # 0x40
'bbog',    # 0x41
'bbogg',    # 0x42
'bbogs',    # 0x43
'bbon',    # 0x44
'bbonj',    # 0x45
'bbonh',    # 0x46
'bbod',    # 0x47
'bbol',    # 0x48
'bbolg',    # 0x49
'bbolm',    # 0x4a
'bbolb',    # 0x4b
'bbols',    # 0x4c
'bbolt',    # 0x4d
'bbolp',    # 0x4e
'bbolh',    # 0x4f
'bbom',    # 0x50
'bbob',    # 0x51
'bbobs',    # 0x52
'bbos',    # 0x53
'bboss',    # 0x54
'bbong',    # 0x55
'bboj',    # 0x56
'bboc',    # 0x57
'bbok',    # 0x58
'bbot',    # 0x59
'bbop',    # 0x5a
'bboh',    # 0x5b
'bbwa',    # 0x5c
'bbwag',    # 0x5d
'bbwagg',    # 0x5e
'bbwags',    # 0x5f
'bbwan',    # 0x60
'bbwanj',    # 0x61
'bbwanh',    # 0x62
'bbwad',    # 0x63
'bbwal',    # 0x64
'bbwalg',    # 0x65
'bbwalm',    # 0x66
'bbwalb',    # 0x67
'bbwals',    # 0x68
'bbwalt',    # 0x69
'bbwalp',    # 0x6a
'bbwalh',    # 0x6b
'bbwam',    # 0x6c
'bbwab',    # 0x6d
'bbwabs',    # 0x6e
'bbwas',    # 0x6f
'bbwass',    # 0x70
'bbwang',    # 0x71
'bbwaj',    # 0x72
'bbwac',    # 0x73
'bbwak',    # 0x74
'bbwat',    # 0x75
'bbwap',    # 0x76
'bbwah',    # 0x77
'bbwae',    # 0x78
'bbwaeg',    # 0x79
'bbwaegg',    # 0x7a
'bbwaegs',    # 0x7b
'bbwaen',    # 0x7c
'bbwaenj',    # 0x7d
'bbwaenh',    # 0x7e
'bbwaed',    # 0x7f
'bbwael',    # 0x80
'bbwaelg',    # 0x81
'bbwaelm',    # 0x82
'bbwaelb',    # 0x83
'bbwaels',    # 0x84
'bbwaelt',    # 0x85
'bbwaelp',    # 0x86
'bbwaelh',    # 0x87
'bbwaem',    # 0x88
'bbwaeb',    # 0x89
'bbwaebs',    # 0x8a
'bbwaes',    # 0x8b
'bbwaess',    # 0x8c
'bbwaeng',    # 0x8d
'bbwaej',    # 0x8e
'bbwaec',    # 0x8f
'bbwaek',    # 0x90
'bbwaet',    # 0x91
'bbwaep',    # 0x92
'bbwaeh',    # 0x93
'bboe',    # 0x94
'bboeg',    # 0x95
'bboegg',    # 0x96
'bboegs',    # 0x97
'bboen',    # 0x98
'bboenj',    # 0x99
'bboenh',    # 0x9a
'bboed',    # 0x9b
'bboel',    # 0x9c
'bboelg',    # 0x9d
'bboelm',    # 0x9e
'bboelb',    # 0x9f
'bboels',    # 0xa0
'bboelt',    # 0xa1
'bboelp',    # 0xa2
'bboelh',    # 0xa3
'bboem',    # 0xa4
'bboeb',    # 0xa5
'bboebs',    # 0xa6
'bboes',    # 0xa7
'bboess',    # 0xa8
'bboeng',    # 0xa9
'bboej',    # 0xaa
'bboec',    # 0xab
'bboek',    # 0xac
'bboet',    # 0xad
'bboep',    # 0xae
'bboeh',    # 0xaf
'bbyo',    # 0xb0
'bbyog',    # 0xb1
'bbyogg',    # 0xb2
'bbyogs',    # 0xb3
'bbyon',    # 0xb4
'bbyonj',    # 0xb5
'bbyonh',    # 0xb6
'bbyod',    # 0xb7
'bbyol',    # 0xb8
'bbyolg',    # 0xb9
'bbyolm',    # 0xba
'bbyolb',    # 0xbb
'bbyols',    # 0xbc
'bbyolt',    # 0xbd
'bbyolp',    # 0xbe
'bbyolh',    # 0xbf
'bbyom',    # 0xc0
'bbyob',    # 0xc1
'bbyobs',    # 0xc2
'bbyos',    # 0xc3
'bbyoss',    # 0xc4
'bbyong',    # 0xc5
'bbyoj',    # 0xc6
'bbyoc',    # 0xc7
'bbyok',    # 0xc8
'bbyot',    # 0xc9
'bbyop',    # 0xca
'bbyoh',    # 0xcb
'bbu',    # 0xcc
'bbug',    # 0xcd
'bbugg',    # 0xce
'bbugs',    # 0xcf
'bbun',    # 0xd0
'bbunj',    # 0xd1
'bbunh',    # 0xd2
'bbud',    # 0xd3
'bbul',    # 0xd4
'bbulg',    # 0xd5
'bbulm',    # 0xd6
'bbulb',    # 0xd7
'bbuls',    # 0xd8
'bbult',    # 0xd9
'bbulp',    # 0xda
'bbulh',    # 0xdb
'bbum',    # 0xdc
'bbub',    # 0xdd
'bbubs',    # 0xde
'bbus',    # 0xdf
'bbuss',    # 0xe0
'bbung',    # 0xe1
'bbuj',    # 0xe2
'bbuc',    # 0xe3
'bbuk',    # 0xe4
'bbut',    # 0xe5
'bbup',    # 0xe6
'bbuh',    # 0xe7
'bbweo',    # 0xe8
'bbweog',    # 0xe9
'bbweogg',    # 0xea
'bbweogs',    # 0xeb
'bbweon',    # 0xec
'bbweonj',    # 0xed
'bbweonh',    # 0xee
'bbweod',    # 0xef
'bbweol',    # 0xf0
'bbweolg',    # 0xf1
'bbweolm',    # 0xf2
'bbweolb',    # 0xf3
'bbweols',    # 0xf4
'bbweolt',    # 0xf5
'bbweolp',    # 0xf6
'bbweolh',    # 0xf7
'bbweom',    # 0xf8
'bbweob',    # 0xf9
'bbweobs',    # 0xfa
'bbweos',    # 0xfb
'bbweoss',    # 0xfc
'bbweong',    # 0xfd
'bbweoj',    # 0xfe
'bbweoc',    # 0xff
)
