
class FiscalizationException(Exception):
    def __init__(
        self,           # type: FiscalizationException
        error_code,     # type: int
        message,        # type: str
    ):
        # type: (...) -> None

        self.error_code = error_code
        self.message = message
