# This module is provided for backwards compatibility with
# googleapis-common-protos <= 1.52.0, where this import path contained
# all of the message and gRPC definitions.

from google.longrunning.operations_proto_pb2 import *
from google.longrunning.operations_proto_pb2 import _OPERATION
from google.longrunning.operations_proto_pb2 import _OPERATION
from google.longrunning.operations_proto_pb2 import _GETOPERATIONREQUEST
from google.longrunning.operations_proto_pb2 import _LISTOPERATIONSREQUEST
from google.longrunning.operations_proto_pb2 import _LISTOPERATIONSRESPONSE
from google.longrunning.operations_proto_pb2 import _CANCELOPERATIONREQUEST
from google.longrunning.operations_proto_pb2 import _DELETEOPERATIONREQUEST
from google.longrunning.operations_proto_pb2 import _OPERATIONINFO
from google.longrunning.operations_proto_pb2 import _OPERATIONS
from google.longrunning.operations_pb2_grpc import *
