class TPag(object):

    DEBITO = "04"
    CREDITO = "03"
    PIX = "17"
    CARTEIRA_DIGITAL = "18"
    VALE_ALIMENTACAO = "10"
    VALE_REFEICAO = "11"
    VALE_PRESENTE = "12"
    VALE_COMBUSTIVEL = "13"
