class EventToken(object):
    SHOW_CHOICES = "SHOW_CHOICES"
    SHOW_MESSAGE = "SHOW_MESSAGE"
    SHOW_QUESTION = "SHOW_QUESTION"
    SHOW_QRCODE = "SHOW_QRCODE"
    SHOW_INPUT = "SHOW_INPUT"
    SHOW_NUMPAD = "SHOW_NUMPAD"
    CLOSE_DIALOGS = "CLOSE_DIALOGS"
    CLOSE_QRCODE = "CLOSE_QRCODE"
